@@ -1,6 +1,7 @@
 from django.utils.functional import curry, SimpleLazyObject
 
-from django_oauth_twitter import ACCESS_KEY, REQUEST_KEY, USERINFO_KEY
+from django_oauth_twitter import (ACCESS_KEY, REQUEST_KEY, SUCCESS_URL_KEY,
+                                  USERINFO_KEY)
 from django_oauth_twitter.utils import get_user_info
 
 from oauth.oauth import OAuthToken
@@ -41,6 +42,14 @@ def cached_user_info(request, token):
         request.session[USERINFO_KEY] = userinfo.AsJsonString()
     return userinfo
 
+def get_success_url(request, clear=True):
+    session = request.session
+    if SUCCESS_URL_KEY in session:
+        token, url = session[SUCCESS_URL_KEY]
+        if token and str(request.twitter_request_token) == str(token):
+            del session[SUCCESS_URL_KEY]
+            return url
+
 def remove_tokens(request):
     if ACCESS_KEY in request.session:
         del request.session[ACCESS_KEY]
@@ -48,6 +57,8 @@ def remove_tokens(request):
     if REQUEST_KEY in request.session:
         del request.session[REQUEST_KEY]
         request.twitter_request_token = None
+    if SUCCESS_URL_KEY in request.session:
+        del request.session[SUCCESS_URL_KEY]
     if USERINFO_KEY in request.session:
         del request.session[USERINFO_KEY]
         request.twitter_userinfo = None
@@ -56,6 +67,7 @@ def set_access_token(request, access_token):
     request.session[ACCESS_KEY] = access_token.to_string()
     request.twitter_access_token = access_token
 
-def set_request_token(request, request_token):
+def set_request_token(request, request_token, success_url):
     request.session[REQUEST_KEY] = request_token.to_string()
+    request.session[SUCCESS_URL_KEY] = (request_token, success_url)
     request.twitter_request_token = request_token
