@@ -5,14 +5,16 @@ from urllib2 import HTTPError
 
 from django.conf import settings
 from django.contrib.auth.models import AnonymousUser, User
-from django.test import TestCase
 from django.core.urlresolvers import NoReverseMatch, reverse
+import django.template.loader
+from django.test import TestCase
 
 from mocker import Mocker, ANY, MATCH
 
 from oauth.oauth import OAuthToken
 
-from django_oauth_twitter import ACCESS_KEY, REQUEST_KEY, USERINFO_KEY
+from django_oauth_twitter import (ACCESS_KEY, REQUEST_KEY, SUCCESS_URL_KEY,
+                                  USERINFO_KEY)
 from django_oauth_twitter.middleware import (cached_user_info,
                                              set_access_token,
                                              set_request_token)
@@ -39,8 +41,9 @@ class MiddlewareTest(TestCase):
         request = self.mocker.mock()
         request.session[REQUEST_KEY] = TOKEN.to_string()
         request.twitter_request_token = TOKEN
+        request.session[SUCCESS_URL_KEY] = (TOKEN, 'url')
         with self.mocker:
-            set_request_token(request, TOKEN)
+            set_request_token(request, TOKEN, 'url')
 
 
 class UtilsTest(TestCase):
@@ -86,11 +89,34 @@ class OAuthTwitterTest(TestCase):
         self.twitter_user = TwitterUser.objects.create(user=self.twitter,
                                                        twitter_id=1,
                                                        userinfo_json=json)
+        # Replace TEMPLATE lookups
         self.old_template_dirs = settings.TEMPLATE_DIRS
-        settings.TEMPLATE_DIRS = (os.path.join(__file__, 'templates'),)
+        settings.TEMPLATE_DIRS = (os.path.join(os.path.dirname(__file__),
+                                               'templates'),)
+        self.old_template_loaders = settings.TEMPLATE_LOADERS
+        settings.TEMPLATE_LOADERS = (
+            'django.template.loaders.filesystem.load_template_source',
+        )
+        django.template.loader.template_source_loaders = None
+        # Replace LOGIN_URL and LOGIN_REDIRECT_URL
+        self.old_login_url = settings.LOGIN_URL
+        settings.LOGIN_URL = '/login/'
+        self.old_login_redirect_url = settings.LOGIN_REDIRECT_URL
+        settings.LOGIN_REDIRECT_URL = '/home/'
+        self.old_login_redirect_urlname = getattr(settings,
+                                                  'LOGIN_REDIRECT_URLNAME',
+                                                  None)
+        settings.LOGIN_REDIRECT_URLNAME = None
 
     def tearDown(self):
+        # Restore TEMPLATE lookups
         settings.TEMPLATE_DIRS = self.old_template_dirs
+        settings.TEMPLATE_LOADERS = self.old_template_loaders
+        django.template.loader.template_source_loaders = None
+        # Restore LOGIN_URL and LOGIN_REDIRECT_URL
+        settings.LOGIN_URL = self.old_login_url
+        settings.LOGIN_REDIRECT_URL = self.old_login_redirect_url
+        settings.LOGIN_REDIRECT_URLNAME = self.old_login_redirect_urlname
 
     def test_association_anonymous(self):
         # Ensure we need a login.
@@ -105,14 +131,16 @@ class OAuthTwitterTest(TestCase):
         self.assertTrue(self.client.login(username=self.password.username,
                                           password=self.password.username))
         Api = self.mocker.replace(TwitterApi)
+        Api().getRequestToken()
+        self.mocker.result(TOKEN)
         Api(ANY).getAccessToken()
         self.mocker.result(TOKEN)
         userinfo = self.mocker.mock()
         userinfo.id
         self.mocker.result('2')
         userinfo.AsJsonString()
-        self.mocker.result('{}')
-        self.mocker.count(1, 2)
+        self.mocker.result('{"id": 2}')
+        self.mocker.count(1, 3)
         Api(ANY).GetUserInfo()
         self.mocker.result(userinfo)
         Api(ANY).GetUserInfo()
@@ -121,15 +149,17 @@ class OAuthTwitterTest(TestCase):
         self.mocker.result(userinfo)
         with self.mocker:
             # User goes off to Twitter and gets redirected back to callback
+            response = self.client.get(reverse('twitter_signin_associate'))
+            self.assertEqual(response.status_code, 302)
             session = self.client.session
             session[REQUEST_KEY] = TOKEN.to_string()
             session.save()
             response = self.client.get(reverse(oauthtwitter.callback),
                                        {'oauth_token': TOKEN.key},
                                        follow=True)
-            self.assertEqual(response.redirect_chain,
-                [('http://testserver/twitter/associate/?associated=1', 302),
-                 ('http://testserver/twitter/associate/', 302)]
+            self.assertEqual(
+                response.redirect_chain,
+                [('http://testserver/twitter/associate/?associated=1', 302)]
             )
             self.assertEqual(response.status_code, 200)
 
@@ -156,6 +186,8 @@ class OAuthTwitterTest(TestCase):
         self.assertTrue(self.client.login(username=self.twitter.username,
                                           password=self.twitter.username))
         Api = self.mocker.replace(TwitterApi)
+        Api().getRequestToken()
+        self.mocker.result(TOKEN)
         Api(ANY).getAccessToken()
         self.mocker.result(TOKEN)
         userinfo = self.mocker.mock()
@@ -163,11 +195,15 @@ class OAuthTwitterTest(TestCase):
         self.mocker.result('2')
         userinfo.AsJsonString()
         self.mocker.result('{}')
-        self.mocker.count(1, 2)
+        self.mocker.count(1, 3)
+        Api(ANY).GetUserInfo()
+        self.mocker.result(userinfo)
         Api(ANY).GetUserInfo()
         self.mocker.result(userinfo)
         with self.mocker:
             # User goes off to Twitter and gets redirected back to callback
+            response = self.client.get(reverse('twitter_signin_associate'))
+            self.assertEqual(response.status_code, 302)
             session = self.client.session
             session[REQUEST_KEY] = TOKEN.to_string()
             session.save()
@@ -178,19 +214,24 @@ class OAuthTwitterTest(TestCase):
                              '?user=twitter&error=user_already_linked')
 
     def test_association_associated_remove(self):
-        # Associated removes association.
-        self.assertTrue(self.twitter.twitter)
-        self.assertTrue(self.client.login(username=self.twitter.username,
-                                          password=self.twitter.username))
-        response = self.client.post(reverse(oauthtwitter.associate))
-        self.assertEqual(response.status_code, 200)
-        self.assertTrue(TwitterUser.objects.get(user=self.twitter))
-        response = self.client.post(reverse(oauthtwitter.associate),
-                                    {'action': 'remove',
-                                     'twitter_id': '1'})
-        self.assertEqual(response.status_code, 200)
-        self.assertRaises(TwitterUser.DoesNotExist,
-                          TwitterUser.objects.get, user=self.twitter)
+        Api = self.mocker.replace(TwitterApi)
+        userinfo = self.mocker.mock()
+        Api(ANY).GetUserInfo()
+        self.mocker.result(userinfo)
+        with self.mocker:
+            # Associated removes association.
+            self.assertTrue(self.twitter.twitter)
+            self.assertTrue(self.client.login(username=self.twitter.username,
+                                              password=self.twitter.username))
+            response = self.client.post(reverse(oauthtwitter.associate))
+            self.assertEqual(response.status_code, 200)
+            self.assertTrue(TwitterUser.objects.get(user=self.twitter))
+            response = self.client.post(reverse(oauthtwitter.associate),
+                                        {'action': 'remove',
+                                         'twitter_id': '1'})
+            self.assertEqual(response.status_code, 200)
+            self.assertRaises(TwitterUser.DoesNotExist,
+                              TwitterUser.objects.get, user=self.twitter)
 
 
     def test_association_user_revoked(self):
@@ -256,8 +297,10 @@ class OAuthTwitterTest(TestCase):
         request.twitter_access_token = ANY
         request.user
         self.mocker.result(User.objects.get(username='password'))
+        self.mocker.count(1, 2)
         cui(request, TOKEN)
         self.mocker.result(userinfo)
+        self.mocker.count(1, 2)
         userinfo.id
         self.mocker.result(3)
         userinfo.AsJsonString()
@@ -270,10 +313,10 @@ class OAuthTwitterTest(TestCase):
                              reverse(oauthtwitter.register))
             response = o.callback(request)
             self.assertEqual(response['Location'],
-                             '/twitter/associate/?associated=1')
+                             '/home/?associated=1')
 
     def test_signin(self):
-        response = self.client.get(reverse(oauthtwitter.signin))
+        response = self.client.get(reverse('twitter_signin'))
         prefix = 'http://twitter.com/oauth/authenticate?'
         self.assertTrue(response['Location'].startswith(prefix),
                         response['Location'])
@@ -286,7 +329,8 @@ class OAuthTwitterTest(TestCase):
         self.assertTrue(self.client.login(username='password',
                                           password='password'))
         response = self.client.get(reverse(oauthtwitter.register))
-        self.assertTrue(response['Location'], '/home/')
+        self.assertEqual(response['Location'],
+                         'http://testserver%s' % settings.LOGIN_REDIRECT_URL)
         # New user comes in through Twitter
         request = self.mocker.mock()
         request.user
@@ -406,12 +450,16 @@ class OAuthTwitterTest(TestCase):
         self.mocker.result('{}')
         request.twitter_userinfo
         self.mocker.result(userinfo)
+        request.session
+        self.mocker.result({SUCCESS_URL_KEY: (TOKEN, '/home/')})
+        request.twitter_request_token
+        self.mocker.result(TOKEN)
         with self.mocker:
-            response = o._login_and_redirect(request, True)
-            self.assertEqual(response['Location'], '/home/')
-            response = o._login_and_redirect(request, None)
-            self.assertEqual(response['Location'], '/home/')
-            response = o._login_and_redirect(request, twitter)
+            response = o._login_and_redirect(request, user=True)
+            self.assertEqual(response['Location'], settings.LOGIN_REDIRECT_URL)
+            response = o._login_and_redirect(request, user=None)
+            self.assertEqual(response['Location'], settings.LOGIN_URL)
+            response = o._login_and_redirect(request, user=twitter)
             self.assertEqual(response['Location'], '/home/')
             self.assertEqual(str(twitter.twitter.access_token), str(TOKEN))
             self.assertEqual(twitter.twitter.userinfo_json, '{}')
